@@ -36,9 +36,12 @@
 #include <yaz/srw.h>
 #include <yaz/cookie.h>
 #include <yaz/mutex.h>
-#if HAVE_LIBMEMCACHED_MEMCACHED_H
+#if HAVE_LIBMEMCACHED
 #include <libmemcached/memcached.h>
 #endif
+#if HAVE_HIREDIS
+#include <hiredis/hiredis.h>
+#endif
 
 #define SHPTR 1
 
@@ -112,9 +115,12 @@ struct ZOOM_connection_p {
     int log_details;
     int log_api;
     WRBUF saveAPDU_wrbuf;
-#if HAVE_LIBMEMCACHED_MEMCACHED_H
+#if HAVE_LIBMEMCACHED
     memcached_st *mc_st;
 #endif
+#if HAVE_HIREDIS
+    redisContext *redis_c;
+#endif
 };
 
 typedef struct ZOOM_record_cache_p *ZOOM_record_cache;
